"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var rest_1 = __importDefault(require("@octokit/rest"));
var helpTopic_1 = require("../helpTopic");
var types_1 = require("./types");
var GitHubCommand = /** @class */ (function (_super) {
    __extends(GitHubCommand, _super);
    function GitHubCommand(username) {
        var _this = _super.call(this) || this;
        _this.name = 'github';
        _this.description = 'shows GitHub profile information';
        _this.subCommands = {};
        _this.octokit = new rest_1.default();
        _this.username = username;
        _this.helpTopic = new helpTopic_1.HelpTopic(_this, {
            synopsis: 'github'
        });
        _this.subCommands['contribs'] = new GitHubContribsSubCommand(_this.octokit, _this.username);
        _this.subCommands['orgs'] = new GitHubOrgsSubCommand(_this.octokit, _this.username);
        _this.subCommands['repos'] = new GitHubReposSubCommand(_this.octokit, _this.username);
        return _this;
    }
    GitHubCommand.prototype.run = function () {
        var ps = [
            // personal data
            this.octokit
                .users
                .getByUsername({ username: this.username })
                .then(function (res) {
                var data = {
                    name: res.data.name,
                    url: res.data.html_url,
                    followers: res.data.followers,
                    public_repos: res.data.public_repos,
                    private_repos: res.data.total_private_repos
                };
                if (res.data.email) {
                    data.email = res.data.email;
                }
                return data;
            }),
            // organizations
            this.octokit
                .orgs
                .listForUser({ username: this.username })
                .then(function (res) {
                return {
                    organizations: res.data.map(function (org) { return org.login; }).join(', ')
                };
            })
        ];
        return Promise.all(ps).then(function (rs) { return rs.reduce(function (op, on) { return Object.assign(op, on); }, {}); });
    };
    return GitHubCommand;
}(types_1.AsyncCommand));
exports.GitHubCommand = GitHubCommand;
var GitHubContribsSubCommand = /** @class */ (function (_super) {
    __extends(GitHubContribsSubCommand, _super);
    function GitHubContribsSubCommand(octokit, username) {
        var _this = _super.call(this) || this;
        _this.name = 'contribs';
        _this.description = 'shows GitHub contributions';
        _this.subCommands = {};
        _this.octokit = octokit;
        _this.username = username;
        _this.helpTopic = new helpTopic_1.HelpTopic(_this, {
            synopsis: 'github contribs',
        });
        return _this;
    }
    GitHubContribsSubCommand.prototype.run = function () {
        var _this = this;
        /**
         * Contributions are defined as merged pull requests at the moment
         * This so answer it's being used as a guide: https://stackoverflow.com/a/23975976/3879872
         */
        return this.searchClosedPulls()
            .then(function (pulls) {
            return pulls
                .filter(function (pull) { return (pull.repository_url.indexOf("https://api.github.com/repos/" + _this.username + "/") !== 0); })
                .map(function (pull) { return ({
                repo: pull.repository_url.replace('https://api.github.com/repos/', ''),
                url: pull.html_url,
                closed_at: pull.closed_at
            }); });
        });
    };
    GitHubContribsSubCommand.prototype.searchClosedPulls = function (page) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.octokit
                .search
                .issues({ q: "type:pr+is:merged+author:" + _this.username, page: page || 1, per_page: 100, sort: 'updated', order: 'asc' })
                .catch(reject)
                .then(function (res) {
                if (page === undefined && res.data.total_count > 100) {
                    // if we just queried the first page and the results are incomplete
                    // then we create a promise for every remaining page and wait for 
                    // all of them to finish in order to resolve the outer one
                    var pages = Math.ceil(res.data.total_count / 100);
                    var ps = [];
                    for (var i = 1; i < pages; i++) {
                        ps.push(_this.searchClosedPulls(i));
                    }
                    Promise
                        .all(ps)
                        .catch(reject)
                        .then(function (rs) {
                        var items = rs.reduce(function (prev, curr) { return prev.concat(curr); }, res.data.items);
                        resolve(items);
                    });
                }
                else {
                    resolve(res.data.items);
                }
            });
        });
    };
    return GitHubContribsSubCommand;
}(types_1.AsyncCommand));
var GitHubOrgsSubCommand = /** @class */ (function (_super) {
    __extends(GitHubOrgsSubCommand, _super);
    function GitHubOrgsSubCommand(octokit, username) {
        var _this = _super.call(this) || this;
        _this.name = 'orgs';
        _this.description = 'shows GitHub organizations';
        _this.subCommands = {};
        _this.octokit = octokit;
        _this.username = username;
        _this.helpTopic = new helpTopic_1.HelpTopic(_this, {
            synopsis: 'github orgs',
        });
        return _this;
    }
    GitHubOrgsSubCommand.prototype.run = function () {
        return this.octokit
            .orgs
            .listForUser({ username: this.username })
            .then(function (res) {
            return res.data.map(function (org) { return ({
                name: org.login,
                url: org.url.replace('api.github.com', 'github.com'),
            }); });
        });
    };
    return GitHubOrgsSubCommand;
}(types_1.AsyncCommand));
var GitHubReposSubCommand = /** @class */ (function (_super) {
    __extends(GitHubReposSubCommand, _super);
    function GitHubReposSubCommand(octokit, username) {
        var _this = _super.call(this) || this;
        _this.name = 'repos';
        _this.description = 'shows GitHub repositories';
        _this.subCommands = {};
        _this.octokit = octokit;
        _this.username = username;
        _this.helpTopic = new helpTopic_1.HelpTopic(_this, {
            synopsis: 'github repos [-l]',
            options: {
                '-l': 'List all repos written in a specified language'
            },
            examples: [
                {
                    cmd: 'github repos -l C++',
                    description: 'The following command shows all owned GitHub repos written in C++ language'
                }
            ]
        });
        return _this;
    }
    GitHubReposSubCommand.prototype.run = function (args) {
        return this.octokit
            .repos
            .listForUser({ username: this.username, sort: 'updated', direction: 'asc' })
            .then(function (res) {
            var data = res.data.map(function (repo) { return ({
                name: repo.full_name,
                language: repo.language || '',
                stars: repo.stargazers_count,
                forks: repo.forks_count,
                last_updated: repo.updated_at
            }); });
            if (args.l) {
                data = data.filter(function (i) { return i.language.indexOf(args.l) >= 0; });
            }
            return data;
        });
    };
    return GitHubReposSubCommand;
}(types_1.AsyncCommand));
//# sourceMappingURL=GitHubCommand.js.map